==> ?url=gopher%3A%2F%2Fftp.icm.edu.pl%3A70%2F9%2Fvol%2Frzm8%2Falsaplayer%2Falsaplayer-0.99.13.tar.bz2: alsaplayer-0.99.13/app/scopes/monoscope.c <==
/*  monoscope.h
 *  Copyright (C) 1998 Andy Lo A Foe <arloafoe@cs.vu.nl>
 *  Original code by Tinic Uro
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 
#include <pthread.h>
#include <dirent.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <sys/time.h>
#include <time.h>   
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <assert.h>
#include "monoscope.h"
#include "scope_config.h"

static char actEq[257];
static char oldEq[257];
static GdkImage *image = NULL;
static GtkWidget *scope_win = NULL;
static int ready_state = 0;
static pthread_t monoscope_thread;
static pthread_mutex_t monoscope_mutex;
static int is_init = 0;
static int running = 0;

static void monoscope_hide();

static const int default_colors[] = {
    10, 20, 30,
    230, 230, 230
};


void monoscope_set_data(void *audio_buffer, int size)
{
	short *sound = (short *)audio_buffer;
	if (!sound) {
			memset(&actEq, 0, sizeof(actEq));
			return;
	}	
	if (running && size > 256) {
		char *newset = actEq;
        	int bufsize = size/512;
		for (int i=0; i < 256; i++) {
			*newset++=(char)(((int)(*sound)+(int)(*(sound+1)))>>10);
			sound += bufsize;
		}
	}
}


void monoscope32(void *data)
{
	guint32 *loc;
	gint foo;
	char *oldset = oldEq;
	char *newset = actEq;
	guint32 *bits;
	int bar;  
	guint32 colEq[65];
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);
		image = NULL;
	}
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
	bg_color.green = SCOPE_BG_GREEN << 8;
	bg_color.blue = SCOPE_BG_BLUE << 8;
        gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp > 2);
	
	bits = (guint32 *)image->mem;	

	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		usleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}



void monoscope16(void *data)
{
	guint16 *loc;
	gint foo;
	char *oldset = oldEq;
	char *newset = actEq;
	guint16 *bits;
	int bar;  
	guint16 colEq[65];
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);	
		image = NULL;
	}
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
        bg_color.green = SCOPE_BG_GREEN << 8;
        bg_color.blue = SCOPE_BG_BLUE << 8;        
	gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp == 2);
	
	bits = (guint16 *)image->mem;	
	
	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		usleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}


void monoscope8(void *data)
{
	guint8 *loc;
	gint8 foo;
	guint8 colEq[65];
	guint8 *bits;
	char *oldset = oldEq;
	char *newset = actEq;
	int bar;  
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);
		image = NULL;
	}	
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
        bg_color.green = SCOPE_BG_GREEN << 8;
        bg_color.blue = SCOPE_BG_BLUE << 8;        
	gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp == 1);
	
	bits = (guint8 *)image->mem;	

	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		usleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}

static GdkVisual *visual;
static GdkWindow *win;

static void test_cb(GtkWidget *widget, gpointer data)
{
	printf("Woah!\n");
}


void stop_monoscope();

void close_monoscope_window(GtkWidget *widget, GdkEvent *event, gpointer data)
{
        stop_monoscope();
}


GtkWidget *init_monoscope_window()
{
	GtkWidget *monoscope_win;
	GtkStyle *style;

	pthread_mutex_init(&monoscope_mutex, NULL);

	style = gtk_style_new();
	monoscope_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(monoscope_win), "Monoscope");
	gtk_widget_set_usize(monoscope_win, 256,128);
	gtk_window_set_wmclass (GTK_WINDOW(monoscope_win), "Monoscope", "AlsaPlayer");
	gtk_window_set_policy (GTK_WINDOW (monoscope_win), FALSE, FALSE, FALSE);
	style = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(monoscope_win)));
	
	GdkColor *color = &style->bg[GTK_STATE_NORMAL];
	color->red = SCOPE_BG_RED << 8;
	color->blue = SCOPE_BG_BLUE << 8;
	color->green = SCOPE_BG_GREEN << 8;
	gdk_color_alloc(gtk_widget_get_colormap(GTK_WIDGET(monoscope_win)), color);
	gtk_widget_set_style(GTK_WIDGET(monoscope_win), style);
	
	gtk_widget_show(monoscope_win);
	
	win = monoscope_win->window;

	// Signals
		
	gtk_signal_connect(GTK_OBJECT(monoscope_win), "delete_event",
                GTK_SIGNAL_FUNC(close_monoscope_window), monoscope_win);


	// Clear and show the window
	gdk_window_clear(win);
	gdk_window_show(win);
	gdk_flush(); 

	ready_state = 1;

	return monoscope_win;
}


void monoscope_hide()
{
	gint x, y;
	
	if (scope_win) {
		gdk_window_get_root_origin(scope_win->window, &x, &y);
		gtk_widget_hide(scope_win);
		gtk_widget_set_uposition(scope_win, x, y);
	}
}


void stop_monoscope()
{
	running = 0;
}


void run_monoscope(void *data)
{
	nice(SCOPE_NICE); // Be nice to most processes

	GDK_THREADS_ENTER();	
	visual = gdk_window_get_visual(win); 
	GDK_THREADS_LEAVE();

	switch (visual->depth) {
	 case 8:
		monoscope8(win);
		break;
	 case 16:
		monoscope16(win);
		break;
	 case 24:
	 case 32:
		monoscope32(win);
		break;
		
	}
	//delete sub;
	pthread_mutex_unlock(&monoscope_mutex);
}


void start_monoscope(void *data)
{
	if (!is_init) {
		is_init = 1;
		scope_win = init_monoscope_window();
	}
	if (pthread_mutex_trylock(&monoscope_mutex) != 0) {
		printf("monoscope already running\n");
		return;
	}
	gtk_widget_show(scope_win);
	pthread_create(&monoscope_thread, NULL, (void * (*)(void *))run_monoscope, data);
	pthread_detach(monoscope_thread);
}


static int open_monoscope()
{
	return 1;
}

static int init_monoscope()
{
	return 1;
}

static void close_monoscope()
{
}

static int monoscope_running()
{
	return running;
}

scope_plugin monoscope_plugin = {
	{ "Monoscope" },
	{ "Andy Lo A Foe"},
	init_monoscope,
	open_monoscope,
	start_monoscope,
	monoscope_running,
	stop_monoscope,
	close_monoscope,
	monoscope_set_data
};

==> ?url=gopher%3A%2F%2Fftp.icm.edu.pl%3A70%2F9%2Fvol%2Frzm8%2Falsaplayer%2Falsaplayer-0.99.14.tar.bz2: alsaplayer-0.99.14/app/scopes/monoscope.c <==
/*  monoscope.h
 *  Copyright (C) 1998 Andy Lo A Foe <arloafoe@cs.vu.nl>
 *  Original code by Tinic Uro
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 
#include <pthread.h>
#include <dirent.h>
#include <sys/stat.h>
#include <gtk/gtk.h>
#include <sys/time.h>
#include <time.h>   
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <assert.h>
#include "monoscope.h"
#include "scope_config.h"

static char actEq[257];
static char oldEq[257];
static GdkImage *image = NULL;
static GtkWidget *scope_win = NULL;
static int ready_state = 0;
static pthread_t monoscope_thread;
static pthread_mutex_t monoscope_mutex;
static int is_init = 0;
static int running = 0;

static void monoscope_hide();

static const int default_colors[] = {
    10, 20, 30,
    230, 230, 230
};


void monoscope_set_data(void *audio_buffer, int size)
{
	short *sound = (short *)audio_buffer;
	if (!sound) {
			memset(&actEq, 0, sizeof(actEq));
			return;
	}	
	if (running && size > 256) {
		char *newset = actEq;
        	int bufsize = size/512;
		for (int i=0; i < 256; i++) {
			*newset++=(char)(((int)(*sound)+(int)(*(sound+1)))>>10);
			sound += bufsize;
		}
	}
}


void monoscope32(void *data)
{
	guint32 *loc;
	gint foo;
	char *oldset = oldEq;
	char *newset = actEq;
	guint32 *bits;
	int bar;  
	guint32 colEq[65];
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);
		image = NULL;
	}
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
	bg_color.green = SCOPE_BG_GREEN << 8;
	bg_color.blue = SCOPE_BG_BLUE << 8;
        gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp > 2);
	
	bits = (guint32 *)image->mem;	

	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		dosleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}



void monoscope16(void *data)
{
	guint16 *loc;
	gint foo;
	char *oldset = oldEq;
	char *newset = actEq;
	guint16 *bits;
	int bar;  
	guint16 colEq[65];
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);	
		image = NULL;
	}
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
        bg_color.green = SCOPE_BG_GREEN << 8;
        bg_color.blue = SCOPE_BG_BLUE << 8;        
	gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp == 2);
	
	bits = (guint16 *)image->mem;	
	
	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		dosleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}


void monoscope8(void *data)
{
	guint8 *loc;
	gint8 foo;
	guint8 colEq[65];
	guint8 *bits;
	char *oldset = oldEq;
	char *newset = actEq;
	int bar;  
	int i, h;
	GdkWindow *win;
	GdkColormap *c;
	GdkVisual *v;
	GdkGC *gc;
	
	win = (GdkWindow *)data;
	GDK_THREADS_ENTER();
	c = gdk_window_get_colormap(win);
	gc = gdk_gc_new(win);
	v = gdk_window_get_visual(win);

	for (i = 0; i < 32; i++) {
		GdkColor color;
		color.red = (i*8) << 8;
		color.green = 255 << 8;
		color.blue = 0;
	        gdk_color_alloc(c, &color);
		colEq[i] = color.pixel; 
		color.red = 255 << 8;
		color.green = ((31 - i) * 8) << 8;
		color.blue = 0;
		gdk_color_alloc(c, &color);
		colEq[i + 32] = color.pixel;
  	}

	// Create render image
	if (image) {
		gdk_image_destroy(image);
		image = NULL;
	}	
	image = gdk_image_new(GDK_IMAGE_FASTEST, v, 256, 128);
	GdkColor bg_color;
        bg_color.red = SCOPE_BG_RED << 8;
        bg_color.green = SCOPE_BG_GREEN << 8;
        bg_color.blue = SCOPE_BG_BLUE << 8;        
	gdk_color_alloc(c, &bg_color); 
	GDK_THREADS_LEAVE();
	
	assert(image);
	assert(image->bpp == 1);
	
	bits = (guint8 *)image->mem;	

	running = 1;

	while (running) {
		for (int w=0; w < 256 * 128; w++) {
			bits[w] = bg_color.pixel;
		}	
	
		//memset(bits,0x0, 256 * 128 * image->bpp);
		memcpy(oldset,newset,256);
		
		for (i=0; i < 256; i++) {
			foo = oldEq[i];
			bar = (i + ((foo+64) << 8 ));
			if ((bar > 0) && (bar < (256 * 128))) {
				loc = bits + bar;
				if (foo < 0) {
					for (h = 0; h <= (-foo); h++) {
						*loc = colEq[h];
						loc+=256;
					}
				} else {
					for (h = 0; h <= foo; h++) {
						*loc = colEq[h];
						loc-=256;
					}
				}
			}
		}
		GDK_THREADS_ENTER();
		gdk_draw_image(win,gc,image,0,0,0,0,-1,-1); 
		gdk_flush();
		GDK_THREADS_LEAVE();
		dosleep(SCOPE_SLEEP);
	}
	GDK_THREADS_ENTER();
	monoscope_hide();
	GDK_THREADS_LEAVE();
}

static GdkVisual *visual;
static GdkWindow *win;

static void test_cb(GtkWidget *widget, gpointer data)
{
	printf("Woah!\n");
}


void stop_monoscope();

void close_monoscope_window(GtkWidget *widget, GdkEvent *event, gpointer data)
{
        stop_monoscope();
}


GtkWidget *init_monoscope_window()
{
	GtkWidget *monoscope_win;
	GtkStyle *style;

	pthread_mutex_init(&monoscope_mutex, NULL);

	style = gtk_style_new();
	monoscope_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(monoscope_win), "Monoscope");
	gtk_widget_set_usize(monoscope_win, 256,128);
	gtk_window_set_wmclass (GTK_WINDOW(monoscope_win), "Monoscope", "AlsaPlayer");
	gtk_window_set_policy (GTK_WINDOW (monoscope_win), FALSE, FALSE, FALSE);
	style = gtk_style_copy(gtk_widget_get_style(GTK_WIDGET(monoscope_win)));
	
	GdkColor *color = &style->bg[GTK_STATE_NORMAL];
	color->red = SCOPE_BG_RED << 8;
	color->blue = SCOPE_BG_BLUE << 8;
	color->green = SCOPE_BG_GREEN << 8;
	gdk_color_alloc(gtk_widget_get_colormap(GTK_WIDGET(monoscope_win)), color);
	gtk_widget_set_style(GTK_WIDGET(monoscope_win), style);
	
	gtk_widget_show(monoscope_win);
	
	win = monoscope_win->window;

	// Signals
		
	gtk_signal_connect(GTK_OBJECT(monoscope_win), "delete_event",
                GTK_SIGNAL_FUNC(close_monoscope_window), monoscope_win);


	// Clear and show the window
	gdk_window_clear(win);
	gdk_window_show(win);
	gdk_flush(); 

	ready_state = 1;

	return monoscope_win;
}


void monoscope_hide()
{
	gint x, y;
	
	if (scope_win) {
		gdk_window_get_root_origin(scope_win->window, &x, &y);
		gtk_widget_hide(scope_win);
		gtk_widget_set_uposition(scope_win, x, y);
	}
}


void stop_monoscope()
{
	running = 0;
}


void run_monoscope(void *data)
{
	nice(SCOPE_NICE); // Be nice to most processes

	GDK_THREADS_ENTER();	
	visual = gdk_window_get_visual(win); 
	GDK_THREADS_LEAVE();

	switch (visual->depth) {
	 case 8:
		monoscope8(win);
		break;
	 case 16:
		monoscope16(win);
		break;
	 case 24:
	 case 32:
		monoscope32(win);
		break;
		
	}
	//delete sub;
	pthread_mutex_unlock(&monoscope_mutex);
}


void start_monoscope(void *data)
{
	if (!is_init) {
		is_init = 1;
		scope_win = init_monoscope_window();
	}
	if (pthread_mutex_trylock(&monoscope_mutex) != 0) {
		printf("monoscope already running\n");
		return;
	}
	gtk_widget_show(scope_win);
	pthread_create(&monoscope_thread, NULL, (void * (*)(void *))run_monoscope, data);
	pthread_detach(monoscope_thread);
}


static int open_monoscope()
{
	return 1;
}

static int init_monoscope()
{
	return 1;
}

static void close_monoscope()
{
}

static int monoscope_running()
{
	return running;
}

scope_plugin monoscope_plugin = {
	{ "Monoscope" },
	{ "Andy Lo A Foe"},
	init_monoscope,
	open_monoscope,
	start_monoscope,
	monoscope_running,
	stop_monoscope,
	close_monoscope,
	monoscope_set_data
};
